import responses
from dagster_census import CensusOutput, CensusResource

from dagster_census_tests.utils import (
    get_destination_data,
    get_source_data,
    get_sync_data,
    get_sync_run_data,
    get_sync_trigger_data,
)


def test_get_sync():
    census = CensusResource(api_key="foo")
    with responses.RequestsMock() as rsps:
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/syncs/52",
            json=get_sync_data(),
        )
        assert census.get_sync(sync_id="52")  # pyright: ignore[reportArgumentType]


def test_get_source():
    census = CensusResource(api_key="foo")
    with responses.RequestsMock() as rsps:
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/sources/15",
            json=get_source_data(),
        )
        assert census.get_source(source_id="15")  # pyright: ignore[reportArgumentType]


def test_get_destination():
    census = CensusResource(api_key="foo")
    with responses.RequestsMock() as rsps:
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/destinations/15",
            json=get_destination_data(),
        )
        assert census.get_destination(destination_id="15")  # pyright: ignore[reportArgumentType]


def test_get_sync_run():
    census = CensusResource(api_key="foo")
    with responses.RequestsMock() as rsps:
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/sync_runs/94",
            json=get_sync_run_data(),
        )
        assert census.get_sync_run(sync_run_id="94")  # pyright: ignore[reportArgumentType]


def test_trigger_sync():
    census = CensusResource(api_key="foo")
    with responses.RequestsMock() as rsps:
        rsps.add(
            rsps.POST,
            "https://app.getcensus.com/api/v1/syncs/52/trigger",
            json=get_sync_trigger_data(),
        )
        assert census.trigger_sync(sync_id="52")  # pyright: ignore[reportArgumentType]


def test_trigger_sync_and_poll():
    census = CensusResource(api_key="foo")
    with responses.RequestsMock() as rsps:
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/syncs/52",
            json=get_sync_data(),
        )
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/sources/15",
            json=get_source_data(),
        )
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/destinations/15",
            json=get_destination_data(),
        )
        rsps.add(
            rsps.POST,
            "https://app.getcensus.com/api/v1/syncs/52/trigger",
            json=get_sync_trigger_data(),
        )
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/sync_runs/94",
            json=get_sync_run_data(),
        )
        result = census.trigger_sync_and_poll(sync_id="52", poll_interval=0)  # pyright: ignore[reportArgumentType]
        assert result == CensusOutput(
            sync_run=get_sync_run_data()["data"],
            source=get_source_data()["data"],
            destination=get_destination_data()["data"],
        )


def test_resource_init():
    cen_resource = CensusResource(api_key="foo")

    assert type(cen_resource) is CensusResource
    assert cen_resource.api_base_url == "https://app.getcensus.com/api/v1"



import asyncio
from collections.abc import Iterator
from contextlib import contextmanager
from typing import Any

import pytest
import responses
from dagster import AssetKey
from dagster._core.definitions.definitions_class import Definitions
from dagster._utils.env import environ
from dagster._utils.test.definitions import scoped_definitions_load_context
from dagster.components.testing.utils import create_defs_folder_sandbox
from dagster_census import CensusComponent
from dagster_shared.merger import deep_merge_dicts

# Test constants
TEST_API_KEY = "test_api_key"
TEST_SYNC_ID_1 = 61
TEST_SYNC_ID_2 = 62
TEST_SYNC_NAME_1 = "test_sync_1"
TEST_SYNC_NAME_2 = "test_sync_2"
TEST_SOURCE_ID = 15
TEST_DESTINATION_ID = 15

CENSUS_API_BASE_URL = "https://app.getcensus.com/api/v1"


def get_sample_sync_data(sync_id: int, sync_name: str) -> dict[str, Any]:
    """Returns sample sync data for testing."""
    return {
        "id": sync_id,
        "label": sync_name,
        "resource_identifier": f"sync_{sync_id}",
        "schedule_frequency": "never",
        "schedule_day": None,
        "schedule_hour": None,
        "schedule_minute": None,
        "created_at": "2021-10-22T00:40:11.246Z",
        "updated_at": "2021-10-22T00:43:44.173Z",
        "operation": "upsert",
        "paused": False,
        "status": "Ready",
        "lead_union_insert_to": None,
        "trigger_on_dbt_cloud_rebuild": False,
        "field_behavior": "specific_properties",
        "field_normalization": None,
        "source_attributes": {
            "connection_id": TEST_SOURCE_ID,
            "object": {
                "type": "model",
                "id": 15,
                "name": "test_model",
                "created_at": "2021-10-11T20:52:58.293Z",
                "updated_at": "2021-10-14T23:15:18.508Z",
                "query": "select email, name from users",
            },
        },
        "destination_attributes": {"connection_id": TEST_DESTINATION_ID, "object": "user"},
        "mappings": [
            {
                "from": {"type": "column", "data": "EMAIL"},
                "to": "external_id",
                "is_primary_identifier": True,
                "generate_field": False,
                "preserve_values": False,
                "operation": None,
            },
            {
                "from": {
                    "type": "constant_value",
                    "data": {"value": "test", "basic_type": "text"},
                },
                "to": "first_name",
                "is_primary_identifier": False,
                "generate_field": False,
                "preserve_values": False,
                "operation": None,
            },
        ],
    }


def get_sample_syncs_response() -> dict[str, Any]:
    """Returns a paginated response for syncs list."""
    return {
        "status": "success",
        "data": [
            get_sample_sync_data(TEST_SYNC_ID_1, TEST_SYNC_NAME_1),
            get_sample_sync_data(TEST_SYNC_ID_2, TEST_SYNC_NAME_2),
        ],
        "pagination": {
            "page": 1,
            "per_page": 25,
            "total": 2,
            "last_page": 1,
        },
    }


@contextmanager
def setup_census_component(
    defs_yaml_contents: dict[str, Any],
) -> Iterator[tuple[CensusComponent, Definitions]]:
    """Sets up a components project with a Census component based on provided params."""
    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=CensusComponent, defs_yaml_contents=defs_yaml_contents
        )
        with (
            scoped_definitions_load_context(),
            sandbox.load_component_and_build_defs(defs_path=defs_path) as (
                component,
                defs,
            ),
        ):
            assert isinstance(component, CensusComponent)
            asyncio.run(component.refresh_state(sandbox.project_root))

        with (
            scoped_definitions_load_context(),
            sandbox.load_component_and_build_defs(defs_path=defs_path) as (
                component,
                defs,
            ),
        ):
            assert isinstance(component, CensusComponent)
            yield component, defs


BASIC_CENSUS_COMPONENT_BODY = {
    "type": "dagster_census.CensusComponent",
    "attributes": {
        "workspace": {
            "api_key": "{{ env.CENSUS_API_KEY }}",
        },
    },
}


@pytest.fixture(name="base_api_mocks")
def base_api_mocks_fixture() -> Iterator[responses.RequestsMock]:
    """Base API mocks fixture."""
    with responses.RequestsMock() as response:
        yield response


@pytest.fixture(name="fetch_workspace_data_api_mocks")
def fetch_workspace_data_api_mocks_fixture(
    base_api_mocks: responses.RequestsMock,
) -> Iterator[responses.RequestsMock]:
    """Fixture that mocks Census API calls for fetching workspace data."""
    base_api_mocks.add(
        method=responses.GET,
        url=f"{CENSUS_API_BASE_URL}/syncs",
        json=get_sample_syncs_response(),
        status=200,
    )
    yield base_api_mocks


def test_basic_component_load(
    fetch_workspace_data_api_mocks: responses.RequestsMock,
) -> None:
    """Test basic component loading with workspace data."""
    with (
        environ(
            {
                "CENSUS_API_KEY": TEST_API_KEY,
            }
        ),
        setup_census_component(
            defs_yaml_contents=deep_merge_dicts(
                BASIC_CENSUS_COMPONENT_BODY,
                {"attributes": {"defs_state": {"management_type": "LOCAL_FILESYSTEM"}}},
            ),
        ) as (
            _component,
            defs,
        ),
    ):
        assert defs.resolve_asset_graph().get_all_asset_keys() == {
            AssetKey(TEST_SYNC_NAME_1),
            AssetKey(TEST_SYNC_NAME_2),
        }


@pytest.mark.parametrize(
    "sync_selector, num_assets",
    [
        ({"by_name": [TEST_SYNC_NAME_1]}, 1),
        ({"by_id": [TEST_SYNC_ID_1]}, 1),
        ({"by_id": []}, 0),
        ({"by_name": [TEST_SYNC_NAME_1, TEST_SYNC_NAME_2]}, 2),
        ({"by_name": [TEST_SYNC_NAME_1, "nonexistent_sync"]}, 1),
    ],
    ids=[
        "single_sync_by_name",
        "single_sync_by_id",
        "no_syncs",
        "multiple_syncs",
        "partial_match",
    ],
)
def test_basic_component_filter(
    fetch_workspace_data_api_mocks: responses.RequestsMock,
    sync_selector: dict[str, Any],
    num_assets: int,
) -> None:
    """Test component filtering with sync selectors."""
    with (
        environ(
            {
                "CENSUS_API_KEY": TEST_API_KEY,
            }
        ),
        setup_census_component(
            defs_yaml_contents=deep_merge_dicts(
                BASIC_CENSUS_COMPONENT_BODY,
                {"attributes": {"sync_selector": sync_selector}},
            ),
        ) as (
            _component,
            defs,
        ),
    ):
        assert len(defs.resolve_asset_graph().get_all_asset_keys()) == num_assets



from dagster.components.component.component_scaffolder import Scaffolder
from dagster.components.component_scaffolding import scaffold_component
from dagster.components.scaffold.scaffold import ScaffoldRequest
from pydantic import BaseModel


class CensusScaffolderParams(BaseModel):
    api_key: str | None = None


class CensusComponentScaffolder(Scaffolder[CensusScaffolderParams]):
    @classmethod
    def get_scaffold_params(cls) -> type[CensusScaffolderParams]:
        return CensusScaffolderParams

    def scaffold(self, request: ScaffoldRequest[CensusScaffolderParams]) -> None:
        scaffold_component(
            request,
            {
                "workspace": {
                    "api_key": "{{ env.CENSUS_API_KEY }}",
                }
            },
        )



from collections.abc import Mapping
from typing import Any

import dagster as dg
from dagster._core.definitions.metadata.metadata_set import NamespacedMetadataSet
from dagster._record import record
from dagster_shared.serdes import whitelist_for_serdes


@whitelist_for_serdes
@record
class CensusSync:
    """Represents a Census sync, based on data as returned from the API."""

    id: int
    name: str
    source_id: int
    destination_id: int
    mappings: list[dict[str, Any]]


@whitelist_for_serdes
@record
class CensusWorkspaceData:
    """A record representing all content in a Census workspace.
    Provided as context for the translator so that it can resolve dependencies between content.
    """

    syncs: list[CensusSync]

    @property
    def syncs_by_id(self) -> Mapping[int, CensusSync]:
        """Returns a mapping of sync IDs to CensusSync objects."""
        return {sync.id: sync for sync in self.syncs}


def generate_table_schema(sync_mappings_props: list[dict[str, Any]]) -> dg.TableSchema:
    return dg.TableSchema(
        columns=sorted(
            [
                dg.TableColumn(name=mapping["to"], type=mapping.get("field_type", "unknown"))
                for mapping in sync_mappings_props
            ],
            key=lambda col: col.name,
        )
    )


class CensusMetadataSet(NamespacedMetadataSet):
    sync_id: int
    sync_name: str
    source_id: int
    destination_id: int

    @classmethod
    def namespace(cls) -> str:
        return "dagster-census"



import responses
from dagster import job, op
from dagster_census import CensusOutput, CensusResource, census_trigger_sync_op

from dagster_census_tests.utils import (
    get_destination_data,
    get_source_data,
    get_sync_data,
    get_sync_run_data,
    get_sync_trigger_data,
)


def test_census_trigger_sync_op():
    cen_resource = CensusResource(api_key="foo")

    @op
    def foo_op():
        pass

    @job(
        resource_defs={"census": cen_resource},
        config={
            "ops": {
                "census_trigger_sync_op": {
                    "config": {
                        "sync_id": 52,
                        "poll_interval": 0,
                        "poll_timeout": 10,
                    }
                }
            }
        },
    )
    def census_sync_job():
        census_trigger_sync_op(start_after=foo_op())

    with responses.RequestsMock() as rsps:
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/syncs/52",
            json=get_sync_data(),
        )
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/sources/15",
            json=get_source_data(),
        )
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/destinations/15",
            json=get_destination_data(),
        )
        rsps.add(
            rsps.POST,
            "https://app.getcensus.com/api/v1/syncs/52/trigger",
            json=get_sync_trigger_data(),
        )
        rsps.add(
            rsps.GET,
            "https://app.getcensus.com/api/v1/sync_runs/94",
            json=get_sync_run_data(),
        )

        result = census_sync_job.execute_in_process()
        assert result.output_for_node("census_trigger_sync_op") == CensusOutput(
            sync_run=get_sync_run_data()["data"],
            source=get_source_data()["data"],
            destination=get_destination_data()["data"],
        )



def get_sync_data():
    return {
        "status": "success",
        "data": {
            "id": 61,
            "label": None,
            "schedule_frequency": "never",
            "schedule_day": None,
            "schedule_hour": None,
            "schedule_minute": None,
            "created_at": "2021-10-22T00:40:11.246Z",
            "updated_at": "2021-10-22T00:43:44.173Z",
            "operation": "upsert",
            "paused": False,
            "status": "Ready",
            "lead_union_insert_to": None,
            "trigger_on_dbt_cloud_rebuild": False,
            "field_behavior": "specific_properties",
            "field_normalization": None,
            "source_attributes": {
                "connection_id": 15,
                "object": {
                    "type": "model",
                    "id": 15,
                    "name": "braze_test",
                    "created_at": "2021-10-11T20:52:58.293Z",
                    "updated_at": "2021-10-14T23:15:18.508Z",
                    "query": "select email, cast('random' as VARCHAR(2000)) as random_prop",
                },
            },
            "destination_attributes": {"connection_id": 15, "object": "user"},
            "mappings": [
                {
                    "from": {"type": "column", "data": "EMAIL"},
                    "to": "external_id",
                    "is_primary_identifier": True,
                    "generate_field": False,
                    "preserve_values": False,
                    "operation": None,
                },
                {
                    "from": {
                        "type": "constant_value",
                        "data": {"value": "test", "basic_type": "text"},
                    },
                    "to": "first_name",
                    "is_primary_identifier": False,
                    "generate_field": False,
                    "preserve_values": False,
                    "operation": None,
                },
            ],
        },
    }


def get_source_data():
    return {
        "status": "success",
        "data": {
            "id": 15,
            "name": "Snowflake - xxxxxxx.us-east-1",
            "label": None,
            "type": "snowflake",
            "last_test_succeeded": None,
            "last_tested_at": None,
            "connection_details": {
                "account": "xxxxxxx.us-east-1",
                "user": "DEV",
                "warehouse": "TEST",
                "use_keypair": False,
            },
            "read_only_connection": False,
        },
    }


def get_destination_data():
    return {
        "status": "success",
        "data": {
            "id": 15,
            "name": "Braze",
            "connection_details": {"instance_url": "https://rest.iad-03.braze.com"},
            "objects": [
                {
                    "label": "User",
                    "full_name": "user",
                    "allow_custom_fields": True,
                    "allow_case_sensitive_field_names": True,
                },
                {
                    "label": "Event",
                    "full_name": "event",
                    "allow_custom_fields": True,
                    "allow_case_sensitive_field_names": True,
                },
            ],
        },
    }


def get_sync_run_data():
    return {
        "status": "success",
        "data": {
            "id": 94,
            "sync_id": 52,
            "source_record_count": 1,
            "records_processed": 1,
            "records_updated": 1,
            "records_failed": 0,
            "records_invalid": 0,
            "created_at": "2021-10-20T02:51:07.546Z",
            "updated_at": "2021-10-20T02:52:29.236Z",
            "completed_at": "2021-10-20T02:52:29.234Z",
            "scheduled_execution_time": None,
            "error_code": None,
            "error_message": None,
            "error_detail": None,
            "status": "completed",
            "canceled": False,
            "full_sync": True,
            "sync_trigger_reason": {
                "ui_tag": "Manual",
                "ui_detail": "Manually triggered by test@getcensus.com",
            },
        },
    }


def get_sync_trigger_data():
    return {"status": "success", "data": {"sync_run_id": 94}}
